#!/usr/bin/env python3


if __name__ == '__main__':
    raise NotImplementedError('Please import Watch_Path instead')


import os
from collections import Iterator


class Watch_Path(dict, Iterator):
    """
    Python 2/3 compatible iterator that watches `path`
    for modified time changes and calls `callback`
    """

    def __init__(self, path, callback, **kwargs):
        # super(Watch_Path, self).__init__(**kwargs)
        self.update(time_stamp = self.file_modified_time(path),
                    path = path,
                    callback = callback)

        self.update(callback_kwargs = kwargs)

    def __iter__(self):
        return self

    def throw(self, type = None, traceback = None):
        raise StopIteration

    @staticmethod
    def file_modified_time(path):
        return os.stat(path).st_mtime

    def next(self):
        """
        When `self['time_stamp']` does not equal `new_time_stamp`
        calls and returns results of `self['callback']` function
        with argument similar to...

            path = self['path']
            time_stamp = self['time_stamp']
        """
        try:
            new_time_stamp = self.file_modified_time(self['path'])
        except OSError as e:
            print(e)
            self.throw(GeneratorExit)

        if new_time_stamp != self['time_stamp']:
            self['time_stamp'] = new_time_stamp
            return self['callback'](path = self['path'],
                                    time_stamp = new_time_stamp,
                                    **self['callback_kwargs'])

    __next__ = next



#!/usr/bin/env python3


from argparse import ArgumentParser
from os.path import basename
from subprocess import (Popen, PIPE)
from sys import argv
from time import sleep


from watch_path import Watch_Path


__license__ = """
Inject JavaScript within PDF document body
Copyright (C) 2020 S0AndS0

This program is free software: you can redistribute it and/or modify
it under the terms of the GNU Affero General Public License as published
by the Free Software Foundation, version 3 of the License.

This program is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU Affero General Public License for more details.

You should have received a copy of the GNU Affero General Public License
along with this program.  If not, see <https://www.gnu.org/licenses/>.
"""


def subprocess_callback(**kwargs):
    """
    """
    if kwargs.get('verbose') > 0:
        print("commands -> {}".format(kwargs['commands']))

    process = Popen(kwargs['commands'], stdout = PIPE, stderr = PIPE, shell = True)
    stdout, stderr = process.communicate()
    if stderr:
        if kwargs.get('decode'):
            raise Exception(stderr.decode(kwargs['decode']))

        raise Exception(stderr)

    if stdout:
        return {"stdout": stdout, "returncode": process.returncode}

    return {"returncode": process.returncode}


arg_parser = ArgumentParser(
    prog = basename(argv[0]),
    usage = '%(prog)s --file "file.txt" --command "cat file.txt"',
    epilog = 'For more projects see: https://github.com/S0AndS0')

arg_parser.add_argument('--path', '--file', '--file-path',
                        help = 'File or directory path to watch for modify time changes',
                        required = True)

arg_parser.add_argument('--commands', '--command',
                        help = 'Command to run when file changes are detected',
                        required = True)

arg_parser.add_argument('--sleep',
                        help = 'Seconds to sleep between checking modify time of `--file`',
                        required = False,
                        default = 1.0,
                        type = float)

arg_parser.add_argument('--decode',
                        help = 'Type of decoding to preform on stdout, eg. "utf-8"',
                        required = False,
                        type = str)

arg_parser.add_argument('--verbose', '-v',
                        help = 'Loudness of this script',
                        action = 'count',
                        default = 0)


args = vars(arg_parser.parse_args())
args.update(callback = subprocess_callback)

path_watcher = Watch_Path(**args)

decode = args.get('decode')


def main():
    try:
        for callback_results in path_watcher:
            if callback_results:
                if args.get('verbose') > 0:
                    print("callback_results['returncode'] ->", callback_results['returncode'])
                    print("callback_results['stdout']...")

                if decode:
                    print(callback_results['stdout'].decode(decode))
                else:
                    print(callback_results['stdout'])

            sleep(args.get('sleep', 1.0))

    except KeyboardInterrupt:
        print('Stopping watcher and exiting...')



#!/usr/bin/env python3


from setuptools import (find_packages, setup)


with open(".github/README.md", "r") as fh:
    long_description = fh.read()


setup(
    name = "watch_path",
    version = "0.1.0",
    author = "S0AndS0",
    author_email = "StrangerThanBland@gmail.com",
    description = "Simple wrapper for `os.stat`, calls callback function time-stamp changes",
    long_description = long_description,
    long_description_content_type = "text/markdown",
    url = "https://github.com/python-utilities/watch_path",
    packages = find_packages(),
    entry_points = {
        'console_scripts': [
            'watch_path = watch_path.cli:main'
        ],
    },
    classifiers = [
        'Development Status :: 4 - Beta',
        'Intended Audience :: Developers',
        'Topic :: Software Development',
        'Programming Language :: Python :: 3',
        'License :: OSI Approved :: GNU Affero General Public License v3',
        'Operating System :: POSIX :: Linux',
    ],
)
